@@ -494,6 +494,11 @@ struct SignatureParsingContext {
   OP *named_elem_defops; /* OP_LINESEQ of those named elements that have defaulting expressions */
   HV *named_details;     /* SV ptrs to NamedParamDetails of every named parameter */
   OP *slurpy_elem;
+
+  /* Counters that replace what PL_parser->sig_* used to be */
+  IV n_elems;
+  IV n_optelems;
+  char slurpy_sigil;
 };
 
 static void free_parsing_ctx(pTHX_ void *_sigctx)
@@ -544,9 +549,9 @@ static void S_sigctx_add_param(pTHX_ struct SignatureParsingContext *sigctx, str
       sigctx->positional_elems = op_append_list(OP_LINESEQ, sigctx->positional_elems,
           newSTATEOP(0, NULL, paramctx->op));
 
-    PL_parser->sig_elems++;
+    sigctx->n_elems++;
     if(paramctx->defop)
-      PL_parser->sig_optelems++;
+      sigctx->n_optelems++;
   }
   else {
     /* The final slurpy */
@@ -555,7 +560,7 @@ static void S_sigctx_add_param(pTHX_ struct SignatureParsingContext *sigctx, str
     if(paramctx->varop)
       sigctx->slurpy_elem = newSTATEOP(0, NULL, paramctx->varop);
 
-    PL_parser->sig_slurpy = paramctx->sigil;
+    sigctx->slurpy_sigil = paramctx->sigil;
   }
 }
 
@@ -564,7 +569,8 @@ void XPS_signature_add_param(pTHX_ struct XSParseSublikeContext *ctx, struct XPS
   /* We know that ctx is really a struct XPSContextWithPointer */
   struct SignatureParsingContext *sigctx = ((struct XPSContextWithPointer *)ctx)->sigctx;
 
-  if(details->ver != XSPARSESUBLIKE_ABI_VERSION)
+  /* Was added in version 7 and so far remains unchanged */
+  if(details->ver < 7 || details->ver > XSPARSESUBLIKE_ABI_VERSION)
     croak("ABI version mismatch in .ver of XPSSignatureParamDetails structure passed to xps_signature_add_param()");
 
   struct XPSSignatureParamContext paramctx = {
@@ -579,7 +585,7 @@ void XPS_signature_add_param(pTHX_ struct XSParseSublikeContext *ctx, struct XPS
   assert(padname_sigil == details->sigil);
   PERL_UNUSED_VAR(padname_sigil);
 
-  paramctx.varop = newUNOP_AUX(OP_ARGELEM, 0, NULL, INT2PTR(UNOP_AUX_item *, (PL_parser->sig_elems)));
+  paramctx.varop = newUNOP_AUX(OP_ARGELEM, 0, NULL, INT2PTR(UNOP_AUX_item *, (sigctx->n_elems)));
   switch(details->sigil) {
     case '$': paramctx.varop->op_private |= OPpARGELEM_SV; break;
     case '@': paramctx.varop->op_private |= OPpARGELEM_AV; break;
@@ -593,6 +599,20 @@ void XPS_signature_add_param(pTHX_ struct XSParseSublikeContext *ctx, struct XPS
   sigctx_add_param(sigctx, &paramctx);
 }
 
+IV XPS_signature_query(pTHX_ struct XSParseSublikeContext *ctx, int q)
+{
+  /* We know that ctx is really a struct XPSContextWithPointer */
+  struct SignatureParsingContext *sigctx = ((struct XPSContextWithPointer *)ctx)->sigctx;
+
+  switch(q) {
+    case 0: return sigctx->n_elems;
+    case 1: return sigctx->n_optelems;
+    case 2: return sigctx->slurpy_sigil;
+    default:
+      croak("ARGH unreachable");
+  }
+}
+
 #define parse_sigelem(sigctx, flags)  S_parse_sigelem(aTHX_ sigctx, flags)
 static void S_parse_sigelem(pTHX_ struct SignatureParsingContext *sigctx, U32 flags)
 {
@@ -651,7 +671,7 @@ static void S_parse_sigelem(pTHX_ struct SignatureParsingContext *sigctx, U32 fl
       /* named params don't get an individual varop */
     }
     else {
-      paramctx.varop = newUNOP_AUX(OP_ARGELEM, 0, NULL, INT2PTR(UNOP_AUX_item *, (parser->sig_elems)));
+      paramctx.varop = newUNOP_AUX(OP_ARGELEM, 0, NULL, INT2PTR(UNOP_AUX_item *, (sigctx->n_elems)));
       paramctx.varop->op_private |= private;
       paramctx.varop->op_targ = paramctx.padix;
     }
@@ -704,7 +724,7 @@ static void S_parse_sigelem(pTHX_ struct SignatureParsingContext *sigctx, U32 fl
     if(paramctx.is_named) {
     }
     else {
-      if(parser->sig_slurpy)
+      if(sigctx->slurpy_sigil)
         yyerror("Slurpy parameters not last");
     }
 
@@ -751,7 +771,7 @@ static void S_parse_sigelem(pTHX_ struct SignatureParsingContext *sigctx, U32 fl
 #endif
 
         OP *defop = (OP *)alloc_LOGOP(OP_ARGDEFELEM, defexpr, LINKLIST(defexpr));
-        defop->op_targ = (PADOFFSET)parser->sig_elems;
+        defop->op_targ = (PADOFFSET)sigctx->n_elems;
         defop->op_private = private;
 
         paramctx.varop->op_flags |= OPf_STACKED;
@@ -768,7 +788,7 @@ static void S_parse_sigelem(pTHX_ struct SignatureParsingContext *sigctx, U32 fl
       }
     }
     else {
-      if(parser->sig_optelems)
+      if(sigctx->n_optelems)
         yyerror("Mandatory parameter follows optional parameter");
 
       if(!paramctx.is_named)
@@ -778,7 +798,7 @@ static void S_parse_sigelem(pTHX_ struct SignatureParsingContext *sigctx, U32 fl
   else {
     if(paramctx.is_named)
       yyerror("Slurpy parameters may not be named");
-    if(parser->sig_slurpy)
+    if(sigctx->slurpy_sigil)
       yyerror("Multiple slurpy parameters not allowed");
 
     if(lex_peek_unichar(0) == '=')
@@ -808,23 +828,16 @@ OP *XPS_parse_subsignature_ex(pTHX_ int flags,
   /* Mostly reconstructed logic from perl 5.28.0's toke.c and perly.y
    */
   yy_parser *parser = PL_parser;
-  struct SignatureParsingContext sigctx = { 0 };
+  struct SignatureParsingContext sigctx_ = { 0 };
+  struct SignatureParsingContext *const sigctx = &sigctx_;
 
   if(ctx)
-    ctx->sigctx = &sigctx;
+    ctx->sigctx = sigctx;
 
   assert((flags & ~(PARSE_SUBSIGNATURE_NAMED_PARAMS|PARSE_SUBSIGNATURE_PARAM_ATTRIBUTES)) == 0);
 
   ENTER;
-  SAVEDESTRUCTOR_X(&free_parsing_ctx, &sigctx);
-
-  SAVEIV(parser->sig_elems);
-  SAVEIV(parser->sig_optelems);
-  SAVEI8(parser->sig_slurpy);
-
-  parser->sig_elems = 0;
-  parser->sig_optelems = 0;
-  parser->sig_slurpy = 0;
+  SAVEDESTRUCTOR_X(&free_parsing_ctx, sigctx);
 
   IV hooki;
   const struct XSParseSublikeHooks *hooks;
@@ -837,7 +850,7 @@ OP *XPS_parse_subsignature_ex(pTHX_ int flags,
 
   while(lex_peek_unichar(0) != ')') {
     lex_read_space(0);
-    parse_sigelem(&sigctx, flags);
+    parse_sigelem(sigctx, flags);
 
     if(PL_parser->error_count) {
       LEAVE;
@@ -873,35 +886,35 @@ endofelems:
       (*hooks->finish_signature)(aTHX_ &(ctx->ctx), hookdata);
   }
 
-  char sig_slurpy = parser->sig_slurpy;
-  if(!sig_slurpy && sigctx.named_details)
-    sig_slurpy = '%';
+  char slurpy_sigil = sigctx->slurpy_sigil;
+  if(!slurpy_sigil && sigctx->named_details)
+    slurpy_sigil = '%';
 
   UNOP_AUX_item *aux = make_argcheck_aux(
-    parser->sig_elems,
-    parser->sig_optelems,
-    sig_slurpy);
+    sigctx->n_elems,
+    sigctx->n_optelems,
+    slurpy_sigil);
 
   OP *checkop = newUNOP_AUX(OP_ARGCHECK, 0, NULL, aux);
 
   OP *ops = op_prepend_elem(OP_LINESEQ, newSTATEOP(0, NULL, NULL),
-      op_prepend_elem(OP_LINESEQ, checkop, sigctx.positional_elems));
+      op_prepend_elem(OP_LINESEQ, checkop, sigctx->positional_elems));
 
-  if(sigctx.named_details) {
-    UV n_params = HvKEYS(sigctx.named_details);
+  if(sigctx->named_details) {
+    UV n_params = HvKEYS(sigctx->named_details);
 
     struct ArgElemsNamedAux *aux = safemalloc(
       sizeof(struct ArgElemsNamedAux) + n_params * sizeof(struct ArgElemsNamedParam)
     );
 
-    aux->start_argix = parser->sig_elems;
+    aux->start_argix = sigctx->n_elems;
     aux->n_params    = n_params;
 
     struct ArgElemsNamedParam *param = &aux->params[0];
 
-    hv_iterinit(sigctx.named_details);
+    hv_iterinit(sigctx->named_details);
     HE *iter;
-    while((iter = hv_iternext(sigctx.named_details))) {
+    while((iter = hv_iternext(sigctx->named_details))) {
       STRLEN namelen;
       const char *namepv = HePV(iter, namelen);
       struct NamedParamDetails *details = (struct NamedParamDetails *)SvPVX(HeVAL(iter));
@@ -926,10 +939,10 @@ endofelems:
 
     OP *argelems_named_op = newUNOP_AUX(OP_CUSTOM, 0, NULL, (UNOP_AUX_item *)aux);
     argelems_named_op->op_ppaddr = &pp_argelems_named;
-    if(PL_parser->sig_slurpy) {
-      if(sigctx.slurpy_elem && sigctx.slurpy_elem->op_type == OP_LINESEQ) {
+    if(sigctx->slurpy_sigil) {
+      if(sigctx->slurpy_elem && sigctx->slurpy_elem->op_type == OP_LINESEQ) {
         /* A real named slurpy variable */
-        OP *o = OpSIBLING(cLISTOPx(sigctx.slurpy_elem)->op_first);
+        OP *o = OpSIBLING(cLISTOPx(sigctx->slurpy_elem)->op_first);
         assert(o);
         assert(o->op_type == OP_ARGELEM);
 
@@ -942,14 +955,14 @@ endofelems:
          * private flags
          */
         argelems_named_op->op_targ    = 0;
-        argelems_named_op->op_private = (PL_parser->sig_slurpy == '%') ? OPpARGELEM_HV :
-                                        (PL_parser->sig_slurpy == '@') ? OPpARGELEM_AV :
-                                                                         0;
+        argelems_named_op->op_private = (sigctx->slurpy_sigil == '%') ? OPpARGELEM_HV :
+                                        (sigctx->slurpy_sigil == '@') ? OPpARGELEM_AV :
+                                                                        0;
       }
 
-      if(sigctx.slurpy_elem) {
-        op_free(sigctx.slurpy_elem);
-        sigctx.slurpy_elem = NULL;
+      if(sigctx->slurpy_elem) {
+        op_free(sigctx->slurpy_elem);
+        sigctx->slurpy_elem = NULL;
       }
     }
 
@@ -958,13 +971,13 @@ endofelems:
     ops = op_append_list(OP_LINESEQ, ops,
         argelems_named_op);
 
-    if(sigctx.named_elem_defops)
+    if(sigctx->named_elem_defops)
       /* TODO: append each elem individually */
       ops = op_append_list(OP_LINESEQ, ops,
-          sigctx.named_elem_defops);
+          sigctx->named_elem_defops);
   }
-  else if(sigctx.slurpy_elem) {
-    ops = op_append_list(OP_LINESEQ, ops, sigctx.slurpy_elem);
+  else if(sigctx->slurpy_elem) {
+    ops = op_append_list(OP_LINESEQ, ops, sigctx->slurpy_elem);
   }
 
   /* a nextstate at the end handles context correctly for an empty
@@ -1021,6 +1034,11 @@ void XPS_signature_add_param(pTHX_ struct XSParseSublikeContext *ctx, struct XPS
 {
 }
 
+IV XPS_signature_query(pTHX_ struct XSParseSublikeContext *ctx, int q)
+{
+  return 0;
+}
+
 void XPS_register_subsignature_attribute(pTHX_ const char *name, const struct XPSSignatureAttributeFuncs *funcs, void *funcdata)
 {
   croak("Custom subroutine signature attributes are not supported on this verison of Perl");
